; ===================================================================
; Project: Help File Check
; Description: Checks TXT files are correctly formatted and referencing existing includes.
; Author: guinness
; Thanks to mLipok (Michal Lipok) for his contribution in developing the concept and debugging.
; ===================================================================

#cs
	Add/remove correct number of \r\n between sections.
	Add/Remove description period
	Check $tag structures are correctly documented.
	Check and format include line e.g. WiNAPI.au3 will be WinAPI.au3.
	Check Example exists and remove accordingly plus report to the end-user.
	Check if parameters in the syntax line match the parameter table and report accordingly to the end-user.
	Check links relative and absolute exist.
	Check the filename matches the title. Exceptions can be included e.g. TCPStartup.
	Check the function's signature includes the same number of variables as documented, including variable and values match.
	Check the includes are documented.
	Check variables conform with the UDF standards.
	Checking whether there are a mismatch with @@End@@.
	Format lines before and after @@ tables.
	Format None to None. ( + period).
	Sort related functions.
	The following script check and/or formats the following:
	Tidy syntax line adding correct number of brackets for optional parameters.
	Tidy white-space.
#ce
; Check formatting of Macros and UDF functions with appending brackets.

#include <File.au3>
#include <String.au3>
#include <WinAPI.au3>

#include 'HelpFileLib.au3' ; HelpFileLib by guinness.
#include 'HelpFileMisc.au3' ; helpFileMisc by guinness.

#Region Help File Constants (Shared between the TXT doc generation and checker scripts)
; Set to the udf_devs\docs directory.
FileChangeDir('..\..\..')

Global Const $AUTOITDOCSOURCE = _GetFullPath('.\autoit\english\txt2htm'), _
		$AUTOITHTMLSOURCE = _GetFullPath('.\autoit\english\html'), _
		$AUTOITXDOCSOURCE = _GetFullPath('.\autoitx\english\txt2htm'), _
		$AUTOITXHTMLSOURCE = _GetFullPath('.\autoitx\english\html\com_interface'), _
		$INCLUDESOURCE = _GetFullPath('..\install\Include'), _
		$EXAMPLES = 'examples', _
		$LIBEXAMPLES = 'libExamples', _
		$TXTFUNCTIONS = 'txtfunctions', _
		$HTMLFUNCTIONS = 'functions', _
		$TXTLIBFUNCTIONS = 'txtlibfunctions', _
		$HTMLLIBFUNCTIONS = 'libfunctions', _
		$TXTKEYWORDS = 'txtKeywords', _
		$HTMLKEYWORDS = 'keywords', _
		$HTMLMACROS = 'macros', _
		$TXTMETHODS = 'txtMethods', _
		$HTMLMETHODS = 'methods', _
		$OPTIONAL = '[optional]'

; These have back/forward slashes appended.
Global Const $HELPMAKE_RELATIVE_TXTKEYWORDS = '..\' & $TXTKEYWORDS & '\', _
		$HELPMAKE_RELATIVE_HTMLKEYWORDS = '../' & $HTMLKEYWORDS & '/', _
		$HELPMAKE_RELATIVE_TXTFUNCTIONS = '..\' & $TXTFUNCTIONS & '\', _
		$HELPMAKE_RELATIVE_HTMLFUNCTIONS = '../' & $HTMLFUNCTIONS & '/', _
		$HELPMAKE_RELATIVE_TXTLIBFUNCTIONS = '..\' & $TXTLIBFUNCTIONS & '\', _
		$HELPMAKE_RELATIVE_HTMLLIBFUNCTIONS = '../' & $HTMLLIBFUNCTIONS & '/', _
		$HELPMAKE_RELATIVE_HTMLMETHODS = '../' & $TXTMETHODS & '/', _
		$HELPMAKE_RELATIVE_TXTMETHODS = '..\' & $HTMLMETHODS & '\'

; Slight speed increase when storing macros to constants.
Global Const $CRLF = @CRLF
Global Const $LF = @LF
Global Const $TAB = @TAB
Global Const $DOTLENGTH = StringLen('.')
; Global $ERROR_NULL_ARRAY[0] ; Empty array constant.
Global Const $HTML_CRLF = '<br />'
Global Const $STR_STRINGSPLIT = BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT)
Global Const $STR_WHITESPACE = BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING)
Global Const $TEMPFILEPATH = _WinAPI_GetTempFileName(@TempDir)

Global Const $PFOP_PATHS = $AUTOITDOCSOURCE & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTFUNCTIONS & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTLIBFUNCTIONS & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTKEYWORDS & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTKEYWORDS & $LF & _
		$AUTOITXDOCSOURCE & '\' & $TXTMETHODS & $LF & _
		$AUTOITHTMLSOURCE & $LF

Global Const $HELP_TABLE_ID = '@@'
#EndRegion Help File Constants (Shared between the TXT doc generation and checker scripts)

#Region Help File Check Constants.
Global Const $HELPCHECK_GUID = '4D3D2540-1B50-11E4-A778-39CCECBA0006'
Global Const $HELPCHECK_DOUBLE_WS = '@DOUBLE_WS'

Global Const $HELPCHECK_ERROR_NONE = 0
Global Enum Step * 2 $HELPCHECK_ERROR_INVALIDSECTION, _
		$HELPCHECK_ERROR_ISINCLUDE_EXISTS, _
		$HELPCHECK_ERROR_ISINCLUDE_NOTEXISTS, _
		$HELPCHECK_ERROR_ISINCLUDE_INVALIDFORMAT, _
		$HELPCHECK_ERROR_SEEALSO_FORMATTED, _
		$HELPCHECK_ERROR_SEEALSO_FORMATTEDWS, _
		$HELPCHECK_ERROR_SYNTAX_INCLUDEFORMAT, _
		$HELPCHECK_ERROR_SYNTAX_PARAMSFORMAT, _
		$HELPCHECK_ERROR_SYNTAX_INVALIDLINE, _
		$HELPCHECK_ERROR_SYNTAX_NOTEXISTS
Global Enum Step * 2 $HELPCHECK_PARAMS_OPTIONAL_NONE, _
		$HELPCHECK_PARAMS_OPTIONAL_NOERROR, _
		$HELPCHECK_PARAMS_OPTIONAL_NOPARAMTABLE, _
		$HELPCHECK_PARAMS_OPTIONAL_MISMATCH, _
		$HELPCHECK_PARAMS_OPTIONAL_INFUNCNOTTABLE, _
		$HELPCHECK_PARAMS_OPTIONAL_INTABLENOTFUNC, _
		$HELPCHECK_PARAMS_OPTIONAL_ISOPTIONALPARAMS, _
		$HELPCHECK_PARAMS_OPTIONAL_ISNOTOPTIONALPARAMS

Global Enum $HELPCHECK_PARAMS_VARIABLE, _
		$HELPCHECK_PARAMS_VALUEORDESC, _
		$HELPCHECK_PARAMS_OPTIONAL, _
		$HELPCHECK_PARAMS_MAX
#EndRegion Help File Check Constants.

; Exit error values.
Global Enum $EXIT_ERROR_NONE, _
		$EXIT_ERROR_USERABORT
OnAutoItExitRegister(_HelpCheck_OnQuit)

Global $g_iReturn = 0
$g_iReturn += _HelpCheck($HELP_TYPE_NATIVE) ; Native
$g_iReturn += _HelpCheck($HELP_TYPE_USERDEFINED) ; UDFs.
$g_iReturn += _HelpCheck($HELP_TYPE_KEYWORD) ; Keywords.
$g_iReturn += _HelpCheck($HELP_TYPE_AUTOITX) ; AutoItX.

#Region Tidy NATIVE/USERDEFINED/KEYWORD TXT Docs
Func _HelpCheck($iType)
	Local $aArray = 0, $aFuncParams = 0, $aIncludeParams = 0, $aParamsTable = 0, _
			$bIsCaseOnly = False, _
			$hHelpFile = 0, _
			$sExampleDirPath = '', _
			$sHelpFileTxtPath = '', _
			$iError = 0, _
			$sFunctionsNotFound = '', $sInclude_After = '', $sInclude_Before = '', $sInfo = '', $sSection = '', $sVariablesNotFound = '', _
			$sParamsCheckingIgnore = FileRead(@ScriptDir & '\parameter_checking.txt') & $CRLF, _
			$vTemp_1 = Null, $vTemp_2 = Null ; This is when a temporary variable is required.

	; Set the filepaths based on the type passed.
	Switch $iType
		Case $HELP_TYPE_AUTOITX
			$sExampleDirPath = $AUTOITXDOCSOURCE & '\' & $EXAMPLES
			$sHelpFileTxtPath = $AUTOITXDOCSOURCE & '\' & $TXTMETHODS

			ConsoleWrite('Scanning methods...' & $CRLF)

		Case $HELP_TYPE_KEYWORD
			$sExampleDirPath = $AUTOITDOCSOURCE & '\' & $EXAMPLES
			$sHelpFileTxtPath = $AUTOITDOCSOURCE & '\' & $TXTKEYWORDS

			ConsoleWrite('Scanning keywords...' & $CRLF)

		Case $HELP_TYPE_NATIVE
			$sExampleDirPath = $AUTOITDOCSOURCE & '\' & $EXAMPLES
			$sHelpFileTxtPath = $AUTOITDOCSOURCE & '\' & $TXTFUNCTIONS

			ConsoleWrite('Scanning functions...' & $CRLF)

		Case $HELP_TYPE_USERDEFINED
			$sExampleDirPath = $AUTOITDOCSOURCE & '\' & $LIBEXAMPLES
			$sHelpFileTxtPath = $AUTOITDOCSOURCE & '\' & $TXTLIBFUNCTIONS

	EndSwitch

	Local $aFilePathTxtList = _FileListToArray($sHelpFileTxtPath, '*.txt', $FLTA_FILES, True)
	If @error Then Return False

	If $iType = $HELP_TYPE_USERDEFINED Then
		ConsoleWrite('Scanning includes directory...' & $CRLF)

		_HelpMisc_CategoriesTOC($aFilePathTxtList, $sHelpFileTxtPath & '\Categories.toc') ; Check Categories.toc file if checking the UDF functions.
		Local $sIncludeParams = ''
		_HelpMisc_IncludeFunctions($aFilePathTxtList, $sIncludeParams) ; Check include functions contain a TXT doc file. $sIncludeParams includes the UDF functions for checking later with the syntax params.

		ConsoleWrite('Scanning UDFs...' & $CRLF)
	EndIf

	Local Const $DOUBLE_WS_ERRORMESSAGE = '! Double white-space in the ' & $HELP_SECTION_NONE & ' section. Please use @TAB if you need 2 or more white-space chars.' & $HELPCHECK_DOUBLE_WS & _
			' marks where the double white-space was.' & $CRLF
	Local Const $HELPFILE_REFERENCE_TYPES = StringReplace($HELP_SECTION_FUNCTION & '|' & $HELP_SECTION_KEYWORD & '|' & _
			$HELP_SECTION_MACRO & '|' & $HELP_SECTION_METHOD & '|' & $HELP_SECTION_OPERATOR & '|' & $HELP_SECTION_USERDEFINED, '#', '')

	Local $iReturn = 0
	For $i = 1 To UBound($aFilePathTxtList) - 1 ; Loop through each TXT doc.
		$hHelpFile = _HelpFile($aFilePathTxtList[$i], $sExampleDirPath)
		Switch $iType
			Case $HELP_TYPE_AUTOITX
				_HelpFile_SetAsAutoItX($hHelpFile)
			Case $HELP_TYPE_KEYWORD
				_HelpFile_SetAsKeyword($hHelpFile)
			Case $HELP_TYPE_NATIVE
				_HelpFile_SetAsNative($hHelpFile)
			Case $HELP_TYPE_USERDEFINED
				_HelpFile_SetAsUserDefined($hHelpFile)
		EndSwitch

		$sInfo = ''
		If _HelpCheck_IsTidy($hHelpFile) Then
			$sInfo &= '! Section white-space and end of line characters formatted.' & $CRLF
		EndIf

		; Check if the structure is ###Reference Type###...###Description###...###Syntax###.
		If Not _HelpFile_IsTag($hHelpFile) And Not StringRegExp(_HelpFile_GetBuildData($hHelpFile), _
				'(?is:^###(?:' & $HELPFILE_REFERENCE_TYPES & ')###.+?\R' & $HELP_SECTION_DESCRIPTION & '.+?\R' & $HELP_SECTION_SYNTAX & ')') Then
			$sInfo &= '! The structure does not follow the format of reference type (function, method, user defined), description and syntax.' & $CRLF
		EndIf

		If Not _HelpFile_IsAutoItX($hHelpFile) And Not _HelpFile_IsTag($hHelpFile) Then
			If Not _HelpCheck_IsInclude($hHelpFile) Then ; Check include line.
				Switch @extended
					Case $HELPCHECK_ERROR_ISINCLUDE_EXISTS
						$sInfo &= '! An example exists but is missing the include line. Please add the appropriate line to the TXT doc.' & $CRLF
					Case $HELPCHECK_ERROR_ISINCLUDE_NOTEXISTS
						$sInfo &= '! An example is missing though has been specified it should exist in the TXT doc. Line has been removed.' & $CRLF
					Case $HELPCHECK_ERROR_ISINCLUDE_INVALIDFORMAT
						$sInfo &= '! Formatting for including an example was incorrect e.g. @@IncludeExample@@ or @@IncludeExample@@ _FuncName. Line has been removed.' & $CRLF
				EndSwitch
			EndIf
		EndIf

		Local $aLinksErrors = _HelpCheck_URLFormatting($hHelpFile)
		If @error Then
			If @extended Then
				$sInfo &= '! Incorrect formatting of links.' & $CRLF
			EndIf
			If UBound($aLinksErrors) Then
				$sInfo &= '! Link errors. The following links are incorrect:' & $CRLF
				For $sLinkError In $aLinksErrors
					$sInfo &= $TAB & $sLinkError & $CRLF
				Next
			EndIf
		EndIf

		; Check if the tables have preceding @@End@@.
		If Not _HelpCheck_IsTablesEnd($hHelpFile) Then
			$sInfo &= '! @@End@@ is missing.' & $CRLF
		EndIf

		; Check if the TXT doc macros i.e. @@IncludeExample@@ are the correct case.
		If _HelpCheck_IsMacrosTidy($hHelpFile) Then
			$sInfo &= '! Incorrect formatting of TXT doc macros.' & $CRLF
		EndIf

		; Check the formatting of tables.
		If _HelpCheck_IsTablesTidy($hHelpFile) Then
			$sInfo &= '! Incorrect formatting of @@ tables.' & $CRLF
		EndIf

		; Change <b> to <strong> etc.
		If _HelpCheck_IsTagsTidy($hHelpFile) Then
			$sInfo &= '! Incorrect formatting of HTML tags.' & $CRLF
		EndIf

		; Check the title of the function matches that of the filename. A list of alternatives can be added to title_checking.txt.
		If Not _HelpFile_IsKeyword($hHelpFile) And Not _HelpCheck_Title($hHelpFile) Then
			$sInfo &= '! The name of the function doesn''t match that of the filepath''s name or possibly the alternative specified in title_checking.txt.' & $CRLF
		EndIf

		; Description - Add period to the end of the line and format double white-space.
		$sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_DESCRIPTION)
		If Not @error Then
			$aArray = StringRegExp($sSection, '(?is)(\V*)(\R.*)?', $STR_REGEXPARRAYGLOBALMATCH)
			If Not @error Then
				$sSection = $aArray[0]
				If _HelpCheck_DescriptionDot($sSection, _HelpFile_IsNative($hHelpFile) Or _HelpFile_IsKeyword($hHelpFile)) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_DESCRIPTION, $sSection & ((UBound($aArray) > 1) ? $aArray[1] : '')) Then
					$sInfo &= '! ' & ((_HelpFile_IsNative($hHelpFile) Or _HelpFile_IsKeyword($hHelpFile)) ? 'Added' : 'Removed') & ' period (.) to the end of the description.' & $CRLF
				EndIf
				; Remove double spacing. Tabs should be used instead.
				If _HelpCheck_IsDoubleSpacing($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_DESCRIPTION, $sSection) Then
					$sInfo &= StringReplace($DOUBLE_WS_ERRORMESSAGE, $HELP_SECTION_NONE, $HELP_SECTION_DESCRIPTION)
				EndIf
			EndIf
		EndIf

		; Check the following if a userdefined TXT doc.
		If _HelpFile_IsUserDefined($hHelpFile) Then
			If _HelpFile_IsTag($hHelpFile) Then ; If a $tag constant.
				$sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_SYNTAX)
				If Not @error Then
					If Not StringRegExp($sSection, '^Global Const \' & _HelpFile_GetFileName($hHelpFile) & ' = ') Then ; If not Global Const $tagConstant = the display an error.
						$sInfo &= '! Tag constant is missing Global Const $tagConstant = or incorrectly formatted.' & $CRLF
					EndIf

					; Syntax - $tag constant line check.
					_HelpCheck_SyntaxTag($sSection, $aFuncParams)
				EndIf
			Else
				; Syntax - Format include line and function line.
				_HelpCheck_Syntax($hHelpFile, $sInclude_Before, $sInclude_After, $aFuncParams)
				$iError = @error
				If $iError Then
					If BitAND($iError, $HELPCHECK_ERROR_SYNTAX_INCLUDEFORMAT) Then $sInfo &= '! Include line formatted from <' & $sInclude_Before & '> to <' & $sInclude_After & '>.' & $CRLF
					If BitAND($iError, $HELPCHECK_ERROR_SYNTAX_NOTEXISTS) Then $sInfo &= '! Include line is referencing an include that doesn''t exist.' & $CRLF
					If BitAND($iError, $HELPCHECK_ERROR_SYNTAX_PARAMSFORMAT) Then $sInfo &= '! Syntax function parameter line was incorrectly formatted.' & $CRLF
				EndIf
			EndIf

			; Parameters - Check if the parameters are declared correctly and/or match the $aFuncParams returned by either _HelpCheck_Syntax() or _HelpCheck_SyntaxTag().
			$vTemp_1 = _HelpFile_GetSection($hHelpFile, (_HelpFile_IsTag($hHelpFile) ? $HELP_SECTION_FIELDS : $HELP_SECTION_PARAMETERS))
			If Not @error Then
				If __HelpCheck_GetParametersTable($vTemp_1, $vTemp_2, $sSection) Then
					If Not _HelpCheck_Optional($sSection, $aFuncParams, $aParamsTable, $sVariablesNotFound, Not _HelpFile_IsTag($hHelpFile)) Then ; Last param is to include $ in the parsing. For $tag constants it's not required.
						$vTemp_1 = ''
						For $iFuncParam = 0 To UBound($aFuncParams) - 1
							Switch $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_OPTIONAL]
								Case $HELPCHECK_PARAMS_OPTIONAL_MISMATCH
									$sInfo &= '! ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & ' not found in the parameter table.' & $CRLF
								Case $HELPCHECK_PARAMS_OPTIONAL_INFUNCNOTTABLE
									$sInfo &= '! ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & ' is optional, but missing ' & $OPTIONAL & ' in the parameter table.' & $CRLF
								Case $HELPCHECK_PARAMS_OPTIONAL_INTABLENOTFUNC
									$sInfo &= '! ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & ' is optional, but missing the Default parameter value.' & $CRLF
							EndSwitch

							If Not _HelpFile_IsTag($hHelpFile) Then
								If Not _HelpCheck_IsParamVariable($aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE]) Then
									$vTemp_1 &= '- Incorrect variable format - ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & $CRLF
								EndIf
							EndIf
						Next
						If $sVariablesNotFound Then
							$sInfo &= '! The following parameters were in the parameter table, but missing in the function syntax. - ' & $sVariablesNotFound & $CRLF
						EndIf

						; If there were variable errors then append to the output.
						If Not ($vTemp_1 == '') Then
							$sInfo &= '! There was an error with the naming convention of the variables in the TXT docs.' & $CRLF & _
									$vTemp_1
						EndIf
					EndIf
				Else
					; When no parameters are present.
					$sSection = $vTemp_1
					If _HelpCheck_IsNone($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_PARAMETERS, $sSection) Then
						$sInfo &= '! Parameter section with no parameter(s) edited from None to None. ( + period).' & $CRLF
					EndIf
				EndIf

				; Check the UDF signature matches the TXT doc.
				If Not StringRegExp($sParamsCheckingIgnore, '(?m:^\b' & _HelpFile_GetFileName($hHelpFile) & '\b)') Then ; If the function doesn't exist in parameter_checking.txt then continue.
					; Check the functions's UDF signature e.g. Func _FuncName(...) matches the TXT doc syntax parameters.
					$aArray = StringRegExp($sIncludeParams, '(?m)(^\b' & _HelpFile_GetFileName($hHelpFile) & '\b\V+)', $STR_REGEXPARRAYGLOBALMATCH) ; $sIncludeParams is a list of all userdefined signatures e.g. _FuncName($vVar, $sVar)
					If Not @error Then ; This can easily be exapanded to check if the default parameters match too. $HELPCHECK_PARAMS_VALUEORDESC would be the column to check.
						$sSection = $aArray[0]
						$vTemp_1 = $aArray[0]
						_HelpCheck_SyntaxParams($vTemp_1, $aIncludeParams) ; Pass $vTemp_1 as the formatted string will not be used in this instance.
						$vTemp_1 = ''
						If UBound($aFuncParams) <> UBound($aIncludeParams) Then ; If the number of parameters fails to match then display an error.
							$sInfo &= '! There was a mismatch with the number of parameters in the includes and those documented in the TXT doc.' & $CRLF & _
									'- This is the UDF signature: ' & $sSection & $CRLF
						Else ; If there was no mismatch then check the variable names are the same and default values match as well.
							$vTemp_1 = ''
							For $iFuncParam = 0 To UBound($aFuncParams) - 1
								If Not ($aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VALUEORDESC] == 'Default') And _
										Not ($aIncludeParams[$iFuncParam][$HELPCHECK_PARAMS_VALUEORDESC] == 'Default') Then ; Is optional and both are not the default keyword.
									If Not ($aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VALUEORDESC] == $aIncludeParams[$iFuncParam][$HELPCHECK_PARAMS_VALUEORDESC]) Then ; Default parameters are mismatched.
										$vTemp_1 &= '- Default parameter mismatch with ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & ' - TXT: ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VALUEORDESC] & ', UDF: ' & $aIncludeParams[$iFuncParam][$HELPCHECK_PARAMS_VALUEORDESC] & $CRLF
									EndIf
								EndIf
								If Not ($aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] == $aIncludeParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE]) Then ; Variable names are mismatched Case-sensitive.
									$bIsCaseOnly = $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] = $aIncludeParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] ; True is $sVar = $svar, therefore case only.
									$vTemp_1 &= '- Variable mismatch ' & ($bIsCaseOnly ? '(CASE-ONLY ISSUE)' : '') & '- TXT: ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & ', UDF: ' & $aIncludeParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & $CRLF
								EndIf

								If Not _HelpFile_IsTag($hHelpFile) Then
									If Not _HelpCheck_IsParamVariable($aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE]) Then
										$vTemp_1 &= '- Incorrect variable naming convention in the TXT doc - ' & $aFuncParams[$iFuncParam][$HELPCHECK_PARAMS_VARIABLE] & $CRLF
									EndIf
								EndIf
							Next
						EndIf
						; Append $vTemp_1 to the output if an error was found.
						If Not ($vTemp_1 == '') Then
							$sInfo &= '! There was an error with the naming convention of the variables or the default values are not matching with the UDFs and the TXT doc.' & $CRLF & _
									$vTemp_1
						EndIf

						$vTemp_1 = ''
						For $iIncludeParam = 0 To UBound($aIncludeParams) - 1 ; Check the include variables are correctly formatted.
							If Not _HelpCheck_IsParamVariable($aIncludeParams[$iIncludeParam][$HELPCHECK_PARAMS_VARIABLE]) Then
								$vTemp_1 &= '- Incorrect variable naming convention in the UDF  - ' & $aIncludeParams[$iIncludeParam][$HELPCHECK_PARAMS_VARIABLE] & $CRLF
							EndIf
						Next
						; Append $vTemp_1 to the output if an error was found.
						If Not ($vTemp_1 == '') Then
							$sInfo &= '! There was an error with the naming convention of the variables in the UDFs.' & $CRLF & _
									$vTemp_1
						EndIf
					EndIf
				EndIf
			EndIf

			; See Also - Check if @@MsdnLink@@ has been used and format the section.
			If Not _HelpCheck_SeeAlso($hHelpFile) Then
				$iError = @error
				If BitAND($iError, $HELPCHECK_ERROR_SEEALSO_FORMATTED) Then $sInfo &= '! The See Also string was formatted incorrectly.' & $CRLF
				If BitAND($iError, $HELPCHECK_ERROR_SEEALSO_FORMATTEDWS) Then $sInfo &= '! Leading and trailing white-space was removed from the See Also section.' & $CRLF
				If BitAND($iError, $HELPCHECK_ERROR_INVALIDSECTION) Then $sInfo &= '! An @MsdnLink@@ has been specified, but no See Also section exists.' & $CRLF
			EndIf
		EndIf

		; Parameters - Format the parameters section for all TXT doc types.
		$sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_PARAMETERS)
		If Not @error Then
			; Format the tabs at the beginning of each line in the table.
			If _HelpCheck_ParametersSpacing($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_PARAMETERS, $sSection) Then
				$sInfo &= '! Parameters tabs were incorrectly formatted.' & $CRLF
			EndIf

			; Remove double spacing. Tabs should be used instead.
			If _HelpCheck_IsDoubleSpacing($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_PARAMETERS, $sSection) Then
				$sInfo &= StringReplace($DOUBLE_WS_ERRORMESSAGE, $HELP_SECTION_NONE, $HELP_SECTION_PARAMETERS)
			EndIf
		EndIf

		; Return - Format None line.
		$sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_RETURNVALUE)
		If Not @error Then
			If _HelpCheck_IsNone($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_RETURNVALUE, $sSection) Then
				$sInfo &= '! Return section with no parameter(s) edited from None to None. ( + period).' & $CRLF
			EndIf

			; Remove double spacing. Tabs should be used instead.
			If _HelpCheck_IsDoubleSpacing($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_RETURNVALUE, $sSection) Then
				$sInfo &= StringReplace($DOUBLE_WS_ERRORMESSAGE, $HELP_SECTION_NONE, $HELP_SECTION_RETURNVALUE)
			EndIf
		EndIf

		; Related - Sort related functions and remove double white-space.
		$sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_RELATED)
		If Not @error Then
			If _HelpCheck_Related($hHelpFile, $sSection, $sFunctionsNotFound) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_RELATED, $sSection) Then
				$sInfo &= '! Related section formatted and sorted.' & $CRLF & (($sFunctionsNotFound = '') ? '' : 'Functions not found: ' & $sFunctionsNotFound & $CRLF)
			EndIf

			; Remove double spacing. Tabs should be used instead.
			If _HelpCheck_IsDoubleSpacing($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_RELATED, $sSection) Then
				$sInfo &= StringReplace($DOUBLE_WS_ERRORMESSAGE, $HELP_SECTION_NONE, $HELP_SECTION_RELATED)
			EndIf
		EndIf

		; Remarks - If there are no remarks ensure None is formatted to None. ( + period). Remove double white-space as well.
		$sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_REMARKS)
		If Not @error Then
			If _HelpCheck_IsNone($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_REMARKS, $sSection) Then
				$sInfo &= '! Remarks section with no remark(s) edited from None to None. ( + period).' & $CRLF
			EndIf

			; Remove double spacing. Tabs should be used instead.
			If _HelpCheck_IsDoubleSpacing($sSection) And _HelpFile_SetSection($hHelpFile, $HELP_SECTION_REMARKS, $sSection) Then
				$sInfo &= StringReplace($DOUBLE_WS_ERRORMESSAGE, $HELP_SECTION_NONE, $HELP_SECTION_REMARKS)
			EndIf
		EndIf

		If $sInfo Then
			$iReturn += 1
			$sInfo = $aFilePathTxtList[$i] & $CRLF & $sInfo & $CRLF
			ConsoleWrite($sInfo)
		EndIf
		_HelpFile_BuildSave($hHelpFile) ; Save the TXT doc.

		$vTemp_1 = Null ; Reset the temporary variable.
		$vTemp_2 = Null ; Reset the temporary variable.
	Next

	Return $iReturn
EndFunc   ;==>_HelpCheck

Func _HelpCheck_DescriptionDot(ByRef $sData, $bIsNative)
	Local $bIsDot = StringRight($sData, StringLen('.')) == '.'
	If $bIsNative Then
		If Not $bIsDot Then $sData &= '.' ; Native functions.
	Else
		If $bIsDot Then $sData = StringTrimRight($sData, StringLen('.')) ; UDF functions.
	EndIf
	Return ($bIsNative ? Not $bIsDot : $bIsDot)
EndFunc   ;==>_HelpCheck_DescriptionDot

Func _HelpCheck_IsDoubleSpacing(ByRef $sData)
	Local $sTabMarker = $HELPCHECK_DOUBLE_WS & '_TAB_' & $HELPCHECK_GUID & '_' & $HELPCHECK_DOUBLE_WS & '_TAB'
	Local $sDataStripped = StringReplace($sData, $TAB, '@TAB_' & $HELPCHECK_GUID & '_@TAB') ; Replace all tabs in the data.
	Local $sDatatrippedOriginal = $sDataStripped ; Store the original contents.
	$sDataStripped = StringRegExpReplace($sDataStripped, '\h{4}', $HELPCHECK_DOUBLE_WS & '_TAB_' & $HELPCHECK_GUID & '_' & $HELPCHECK_DOUBLE_WS & '_TAB') ; Replace white-space of 4 chars.
	$sDataStripped = StringRegExpReplace($sDataStripped, '\h{2,}', $HELPCHECK_DOUBLE_WS & '_' & $HELPCHECK_GUID & '_' & $HELPCHECK_DOUBLE_WS) ; Replace white-space greater than 2 chars.
	Local $bReturn = Not ($sDatatrippedOriginal == $sDataStripped)
	If $bReturn Then ; Strip double white-space in the orginal data.
		$sData = StringRegExpReplace($sData, '\h{4}', $TAB) ; Replace white-space of 4 chars with @TAB.
		$sData = StringReplace($sData, $TAB, $sTabMarker) ; Replace tabs with the tab marker, so as not to strip accidentally.
		$sData = StringRegExpReplace($sData, '\h{2,}', $HELPCHECK_DOUBLE_WS) ; Replace white-space greater than 2 chars with the white-space marker.
		$sData = StringReplace($sData, $sTabMarker, $TAB) ; Replace the tab marker with tabs.
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsDoubleSpacing

Func _HelpCheck_IsInclude(ByRef $hHelpFile) ; Check and remove (if applicable) the include script line.
	Local $bExampleExists = True, $bReturn = True, _
			$iExtended = $HELPCHECK_ERROR_NONE
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_EXAMPLE)
		If @error Then Return $bReturn ; No example section exists.

		; Check if an Au3 script exists one folder up and in the examples folder.
		Local $sExampleFilePath = _HelpMisc_GetIncludeFilePath($hHelpFile)
		$bExampleExists = FileExists($sExampleFilePath) > 0
		$bReturn = StringRegExp($sSection, '(?i:^' & $HELP_TABLE_ID & 'IncludeExample' & $HELP_TABLE_ID & '\h*)') > 0
		If $bReturn Then ; Include section exists.
			If $bExampleExists Then
				; Tidy the include line by removing any preceeding spaces.
				$sSection = StringRegExpReplace($sSection, '(?i:^(' & $HELP_TABLE_ID & 'IncludeExample' & $HELP_TABLE_ID & ')\h*( \w+))', '\1\2')
				_HelpFile_SetSection($hHelpFile, $HELP_SECTION_EXAMPLE, $sSection)
			Else
				; Remove the include line from the TXT doc.
				_HelpFile_RemoveSection($hHelpFile, $HELP_SECTION_EXAMPLE)
				$iExtended = $HELPCHECK_ERROR_ISINCLUDE_NOTEXISTS ; @extended - Removed the line from the TXT doc.
			EndIf
			_HelpFile_BuildSectionData($hHelpFile)
		Else
			If $bExampleExists Then
				$iExtended = $HELPCHECK_ERROR_ISINCLUDE_EXISTS ; @extended - Example exists but no line is included in the TXT doc.
			Else
				_HelpFile_RemoveSection($hHelpFile, $HELP_SECTION_EXAMPLE)
				$iExtended = $HELPCHECK_ERROR_ISINCLUDE_INVALIDFORMAT ; @extended - Removed the line from the TXT doc.
				_HelpFile_BuildSectionData($hHelpFile)
			EndIf
		EndIf
	EndIf
	Return SetExtended($iExtended, $bReturn And $bExampleExists)
EndFunc   ;==>_HelpCheck_IsInclude

Func _HelpCheck_IsMacrosTidy(ByRef $hHelpFile)
	Local $bReturn = False
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $sData = _HelpFile_GetBuildData($hHelpFile)
		Local $aMacros = [ _
				'ControlCommandTable', _
				'End', _
				'IncludeExample', _
				'MsdnLink', _
				'ParamTable', _
				'ReturnTable', _
				'StandardTable', _
				'StandardTable1' _
				]
		For $sMacro In $aMacros
			$sData = StringReplace($sData, $HELP_TABLE_ID & $sMacro & $HELP_TABLE_ID, $HELP_TABLE_ID & $sMacro & $HELP_TABLE_ID)
		Next
		$bReturn = _HelpFile_SetBuildData($hHelpFile, $sData)
		If $bReturn Then
			_HelpFile_BuildSections($hHelpFile)
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsMacrosTidy

Func _HelpCheck_IsNone(ByRef $sData)
	Local $bReturn = False
	If StringStripWS($sData, $STR_STRIPALL) = '' Or _HelpMisc_IsNone($sData) Then
		Local $sDataOriginal = $sData
		$sData = 'None.'
		$bReturn = Not ($sDataOriginal == $sData)
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsNone

Func _HelpCheck_IsParamVariable(ByRef $sVariable)
	Local $bReturn = False
	If StringRegExp($sVariable, '^\$[lw]P') Or StringRegExp($sVariable, '^\$fu[A-Z]') Then $bReturn = True ; Workaround for $wParam and $lParam or function variables.
	If Not $bReturn Then
		; $bReturn = StringRegExp($sVariable, '^\$[adhibfnsvopt](?(?<=i)d?|(?(?<=a)[adhibfnsvo]?))[A-Z]') > 0
		; JPM : fixed $aaX is invalid ; added $paX is a valid ptr to array
		$bReturn = StringRegExp($sVariable, '^\$[adhibfnsvopt](?(?<=i)d?|(?(?<=p)a?[dhibfnsvo]?|(?(?<=a)[dhibfnsvo]?)))[A-Z]') > 0
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsParamVariable

Func _HelpCheck_IsTablesEnd(ByRef $hHelpFile)
	Local $bReturn = True
	If _HelpFile_IsHelpFile($hHelpFile) Then
		StringRegExpReplace(_HelpFile_GetBuildData($hHelpFile), '(?im:^' & $HELP_TABLE_ID & '(?:ControlCommand|Param|Return|Standard)Table(?:1)?' & $HELP_TABLE_ID & '\R)', '')
		Local $iCount = @extended
		If $iCount > 0 Then
			StringRegExpReplace(_HelpFile_GetBuildData($hHelpFile), '(?im:^' & $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID & '\R)', '')
			$bReturn = @extended = $iCount
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsTablesEnd

Func _HelpCheck_IsTablesTidy(ByRef $hHelpFile)
	Local $bReturn = False
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $iExtended = 0, _
				$sData = _HelpFile_GetBuildData($hHelpFile)
		$sData = StringRegExpReplace($sData, '(?i:((' & $HELP_TABLE_ID & '(?:ControlCommand|Param|Return|Standard)Table(?:1)?' & $HELP_TABLE_ID & '))\R{2,})', '\1' & $CRLF)
		$iExtended += @extended
		$sData = StringRegExpReplace($sData, '(?i:\R{2,}' & $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID & ')', $CRLF & $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
		$iExtended += @extended
		If $iExtended > 0 Then
			$bReturn = _HelpFile_SetBuildData($hHelpFile, $sData)
			If $bReturn Then
				_HelpFile_BuildSections($hHelpFile)
			EndIf
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsTablesTidy

Func _HelpCheck_IsTagsTidy(ByRef $hHelpFile)
	Local $bReturn = False
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $iExtended = 0, _
				$sData = _HelpFile_GetBuildData($hHelpFile)
		$sData = StringReplace($sData, '<i>', '<em>', 0, $STR_CASESENSE)
		$iExtended += @extended
		$sData = StringReplace($sData, '</i>', '</em>', 0, $STR_CASESENSE)
		$iExtended += @extended
		$sData = StringReplace($sData, '<b>', '<strong>', 0, $STR_CASESENSE)
		$iExtended += @extended
		$sData = StringReplace($sData, '</b>', '</strong>', 0, $STR_CASESENSE)
		$iExtended += @extended
		If $iExtended > 0 Then
			$bReturn = _HelpFile_SetBuildData($hHelpFile, $sData)
			If $bReturn Then
				_HelpFile_BuildSections($hHelpFile)
			EndIf
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsTagsTidy

Func _HelpCheck_IsTidy(ByRef $hHelpFile)
	Local $bReturn = False
	If _HelpFile_IsHelpFile($hHelpFile) Then
		_HelpFile_BuildSectionData($hHelpFile)
		$bReturn = Not (_HelpFile_GetBuildData($hHelpFile) == _HelpFile_GetData($hHelpFile))
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_IsTidy

Func _HelpCheck_OnQuit()
	; Delete the temporary filepath.
	FileDelete($TEMPFILEPATH)

	Exit $g_iReturn
EndFunc   ;==>_HelpCheck_OnQuit

Func _HelpCheck_Optional(ByRef $sData, ByRef $aFuncParams, ByRef $aParamsTable, ByRef $sVariablesNotFound, $bIsVariable)
	$aParamsTable = 0
	$sVariablesNotFound = ''

	Local $bReturn = False
	Local $aParams = StringRegExp($sData, '(?ms)(^\$' & ($bIsVariable ? '' : '?') & '\w+)(\R.*?(?=\R\$|' & ($bIsVariable ? '' : '\R\w|') & '$))', $STR_REGEXPARRAYGLOBALMATCH)
	If @error Then Return SetError($HELPCHECK_PARAMS_OPTIONAL_NOPARAMTABLE, 0, $bReturn)
	; If Not (UBound($aFuncParams) = (UBound($aParams) / 2)) Then Return SetError($HELPCHECK_PARAMS_OPTIONAL_MISMATCH, 0, $bReturn)

	Local $hVarIndex = 0, $hVarIsOptional = 0
	If _AssociativeArray_Startup($hVarIsOptional, False) And _AssociativeArray_Startup($hVarIndex, False) Then
		For $i = 0 To UBound($aFuncParams) - 1 ; Create associative arrays for the array index value and variable value.
			If Not $hVarIsOptional.Exists($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE]) Then $hVarIsOptional.Item($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE]) = StringStripWS($aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC], $STR_STRIPALL)
			If Not $hVarIndex.Exists($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE]) Then $hVarIndex.Item($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE]) = $i
			$aFuncParams[$hVarIndex.Item($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE])][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_MISMATCH
		Next

		Local Const $OPTIONAL_START_POS = 1
		Local $bIsOptional = False, _
				$sDescription = '', $sVariable = ''
		For $i = 0 To UBound($aParams) - 1 Step 2
			$sVariable = StringStripWS($aParams[$i + $HELPCHECK_PARAMS_VARIABLE], $STR_WHITESPACE)
			$sDescription = StringStripWS($aParams[$i + $HELPCHECK_PARAMS_VALUEORDESC], $STR_WHITESPACE)

			If Not $hVarIsOptional.Exists($sVariable) Then
				$sVariablesNotFound &= $sVariable & ','
				ContinueLoop ; Variable in the parameter table doesn't exist in the syntax function.
			EndIf

			; Check if the parameter is actually optional or not.
			$bIsOptional = StringInStr($sDescription, $OPTIONAL, $STR_CASESENSE, $STR_REPLACE_ONCE) = $OPTIONAL_START_POS
			If $bIsOptional And Not ($hVarIsOptional.Item($sVariable) == '') Then
				$aFuncParams[$hVarIndex.Item($sVariable)][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_NOERROR
			Else ; If optional table and function syntax mismatch.
				If $bIsOptional And $hVarIsOptional.Item($sVariable) == '' Then
					$aFuncParams[$hVarIndex.Item($sVariable)][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_INTABLENOTFUNC
					$bReturn = False
				ElseIf Not $bIsOptional And Not ($hVarIsOptional.Item($sVariable) == '') Then
					$aFuncParams[$hVarIndex.Item($sVariable)][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_INFUNCNOTTABLE
					$bReturn = False
				Else
					$aFuncParams[$hVarIndex.Item($sVariable)][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_NOERROR
				EndIf
			EndIf
		Next
		$aParamsTable = $aParams
		$aParams = 0
		$sVariablesNotFound = StringTrimRight($sVariablesNotFound, StringLen(','))
	EndIf

	Return $bReturn
EndFunc   ;==>_HelpCheck_Optional

Func _HelpCheck_ParametersSpacing(ByRef $sData)
	Return False ; Disable for now.

	Local $bReturn = False, _
			$sSection = '', $sSectionOriginal = ''
	If __HelpCheck_GetParametersTable($sData, $sSectionOriginal, $sSection) Then
		Local $aArray = StringSplit($sSection, $CRLF, $STR_STRINGSPLIT), _
				$bIndent = False
		$sSection = ''
		For $i = 0 To UBound($aArray) - 1
			If Not (StringStripWS($aArray[$i], $STR_STRIPALL) = '') Then
				If StringRegExp($aArray[$i], '^\h+') Then
					$aArray[$i] = StringStripWS($aArray[$i], $STR_WHITESPACE)
					If $bIndent Then
						$aArray[$i] = $TAB & $TAB & $aArray[$i]
					Else
						$aArray[$i] = $TAB & $aArray[$i]
						$bIndent = True
					EndIf
				Else
					$bIndent = False
					$aArray[$i] = StringStripWS($aArray[$i], $STR_WHITESPACE)
				EndIf
			EndIf
			$sSection &= $aArray[$i] & $CRLF
		Next
		$sSection = $HELP_TABLE_ID & 'ParamTable' & $HELP_TABLE_ID & $CRLF & _
				$sSection & _
				$HELP_TABLE_ID & 'End' & $HELP_TABLE_ID
		If Not ($sSection == $sSectionOriginal) Then
			$bReturn = True
			$sData = StringReplace($sData, $sSectionOriginal, $sSection, $STR_REPLACE_ONCE, $STR_CASESENSE)
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_ParametersSpacing

Func _HelpCheck_Related(ByRef $hHelpFile, ByRef $sData, ByRef $sFunctionsNotFound)
	; Set the out parameters.
	$sFunctionsNotFound = ''

	Local $bReturn = False
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $bIsNativeType = _HelpFile_IsAutoItX($hHelpFile) Or _HelpFile_IsNative($hHelpFile) Or _HelpFile_IsKeyword($hHelpFile)
		Local $sDataOriginal = $sData

		If StringStripWS($sData, $STR_STRIPALL) = '' Then
			$sData = 'None.'
			Return Not ($sDataOriginal == $sData)
		EndIf
		If _HelpMisc_IsNone($sData) Then
			$sData = 'None.'
			Return Not ($sDataOriginal == $sData)
		EndIf

		Local $aFuncs = 0
		For $i = 0 To UBound($aFuncs) - 1
			$aFuncs[$i] = StringStripWS($aFuncs[$i], $STR_WHITESPACE)
		Next
		$sData = ''

		_ArraySort($aFuncs)
		$aFuncs = _ArrayUnique($aFuncs)
		If @error Then
			$sData = $sDataOriginal
		Else
			Local $bIsAppend = True, $bIsChecking = False, _
					$iDotLen = StringLen('.'), _
					$sFuncName = ''
			For $i = 1 To UBound($aFuncs) - 1
				$bIsAppend = True
				; Reset the checking variables.
				$bIsChecking = False
				$sFuncName = ''

				; Reset the append variable.
				$bIsAppend = True
				If Not StringInStr($aFuncs[$i], '<a href=') Then ; If it's a link then skip formatting the hyperlink.
					If StringInStr($aFuncs[$i], '...') Then ; GUI management.
						; Do nothing.

					ElseIf StringInStr($aFuncs[$i], '(Option)') Then ; AutoItSetOption.
						; Do nothing.

					ElseIf StringLeft($aFuncs[$i], $iDotLen) == '.' Then ; Native function inside a UDF function.
						$bIsChecking = True
						$sFuncName = StringTrimLeft($aFuncs[$i], $iDotLen) ; Strip the dot.
						If $bIsNativeType Then ; Strip the dot if a native function.
							$aFuncs[$i] = $sFuncName
						EndIf


					ElseIf StringRegExp($aFuncs[$i], '(?:^(?:\w|\_|\$))') Then ; It's a function/keyword.
						$bIsChecking = True
						$sFuncName = $aFuncs[$i]

						; For native functions only where a prefix of a dot is required.
						If Not $bIsNativeType And Not StringRegExp($aFuncs[$i], '(?:^(?:\.|\_|\$))') Then ; If no dot and not a UDF or tag constant.
							$aFuncs[$i] = '.' & $aFuncs[$i]
						EndIf

					Else ; There was an error, which shouldn't happen.
						$bIsAppend = False

					EndIf
				EndIf

				; Related checking.
				If $bIsChecking Then
					$sFuncName = _WinAPI_PathFindOnPath($sFuncName & '.txt', $PFOP_PATHS)
					If @error Then
						; File doesn't exist.
						$bIsAppend = False
						$sFunctionsNotFound &= $aFuncs[$i] & ','
					Else
						; Do nothing.
					EndIf
				EndIf

				If $bIsAppend Then
					$sData &= $aFuncs[$i] & ', '
				EndIf
			Next

			$sData = StringTrimRight($sData, StringLen(', '))
			If StringStripWS($sData, $STR_STRIPALL) = '' Then
				$sData = 'None.'
			EndIf
		EndIf

		$sFunctionsNotFound = StringTrimRight($sFunctionsNotFound, StringLen(','))
		$bReturn = Not ($sDataOriginal == $sData)
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_Related

Func _HelpCheck_SeeAlso(ByRef $hHelpFile)
	Local $bReturn = False, _
			$iError = $HELPCHECK_ERROR_NONE
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_SEEALSO)
		If @error Then ; If no section exists then check the following scenarios.
			If StringInStr(_HelpFile_GetBuildData($hHelpFile), $HELP_TABLE_ID & 'MsdnLink' & $HELP_TABLE_ID) Then ; If @@MsdnLink@@ is used then add a See Also section.
				$iError = $HELPCHECK_ERROR_INVALIDSECTION
			EndIf
		Else
			Local $sSectionOrginal = $sSection

			; Strip white-space after a @@MsdnLink@@ e.g. @@MsdnLink@@    FuncName => @@MsdnLink@@ FuncName.
			$sSection = StringRegExpReplace($sSection, '(?:' & $HELP_TABLE_ID & 'MsdnLink' & $HELP_TABLE_ID & '\h+)', $HELP_TABLE_ID & 'MsdnLink' & $HELP_TABLE_ID & ' ')
			If Not ($sSection == $sSectionOrginal) Then
				$iError = BitOR($iError, $HELPCHECK_ERROR_SEEALSO_FORMATTEDWS)
			EndIf

			Local $aArray = StringSplit($sSection, ', ', $STR_STRINGSPLIT), _
					$sDataOriginal = ''
			$sSection = ''
			For $i = 0 To UBound($aArray) - 1
				$sDataOriginal = $aArray[$i]
				$aArray[$i] = StringStripWS($aArray[$i], $STR_WHITESPACE)
				If Not ($aArray[$i] == $sDataOriginal) Then
					$iError = BitOR($iError, $HELPCHECK_ERROR_SEEALSO_FORMATTEDWS)
				EndIf

				; Re-build the line.
				; This can be either be Win32 API or URL. Generally speaking it will be a Win32 API though there are exceptions.
				If StringRegExp($aArray[$i], '(?:^' & $HELP_TABLE_ID & 'MsdnLink' & $HELP_TABLE_ID & '\h\w+$)') Then ; An @@MsdnLink@@ was found.
					$sSection &= $aArray[$i] & ', '
				Else ; Else it's a URL.
					If _WinAPI_UrlIs($aArray[$i]) Then
						$sSection &= $aArray[$i] & ', '
					EndIf
				EndIf
			Next
			$sSection = StringTrimRight($sSection, 2) ; 2 = StringLen(', '). Strip the last space and comma.
			If Not ($sSection == $sSectionOrginal) Then
				$iError = BitOR($iError, $HELPCHECK_ERROR_SEEALSO_FORMATTED)
				_HelpFile_SetSection($hHelpFile, $HELP_SECTION_SEEALSO, $sSection)
				_HelpFile_BuildSectionData($hHelpFile)
			EndIf
		EndIf
		If $iError > $HELPCHECK_ERROR_NONE Then ; If an error occurred then set the return to false.
			$bReturn = False
		EndIf
	EndIf
	Return SetError($iError, 0, $bReturn)
EndFunc   ;==>_HelpCheck_SeeAlso

Func _HelpCheck_Syntax(ByRef $hHelpFile, ByRef $sInclude_Before, ByRef $sInclude_After, ByRef $aFuncParams)
	; Set the out parameters.
	$aFuncParams = 0
	$sInclude_After = ''
	$sInclude_Before = ''

	Local $bReturn = False, _
			$iError = $HELPCHECK_ERROR_NONE
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local Static $hIncludes = 0
		If Not IsObj($hIncludes) Then
			_AssociativeArray_Startup($hIncludes)
			Local $aIncludes = _FileListToArray($INCLUDESOURCE, '*.au3', $FLTA_FILES, True)
			If Not @error Then
				Local $sFilePath = ''
				For $i = 1 To UBound($aIncludes) - 1
					$sFilePath = _WinAPI_PathStripPath($aIncludes[$i])
					$hIncludes.Item($sFilePath) = $sFilePath
				Next
			EndIf
			$aIncludes = 0
		EndIf

		Local $sSection = _HelpFile_GetSection($hHelpFile, $HELP_SECTION_SYNTAX), _
				$sSectionOrginal = $sSection
		Local Enum $SECTION_INCLUDE, $SECTION_PARAMS, $SECTION_MAX
		Local $aSection = StringSplit($sSection, $CRLF, $STR_STRINGSPLIT) ; Retrieve the section and split and the EOL char.
		If UBound($aSection) <> $SECTION_MAX Then Return SetError($HELPCHECK_ERROR_INVALIDSECTION, 0, $bReturn)

		Local $sIncludePath = StringStripWS(StringRegExp($aSection[$SECTION_INCLUDE] & '#include "' & $HELPFILE_GUID & '"' & $CRLF, _
				'(?i:#include\h*["''<](.*?)["''>]\s*)', $STR_REGEXPARRAYGLOBALMATCH)[0], _
				$STR_WHITESPACE)

		If $sIncludePath == $HELPFILE_GUID Then Return SetError($HELPCHECK_ERROR_SYNTAX_INVALIDLINE, 0, $bReturn)
		If Not $hIncludes.Exists($sIncludePath) Then Return SetError($HELPCHECK_ERROR_SYNTAX_NOTEXISTS, 0, $bReturn)

		$sInclude_After = $hIncludes.Item($sIncludePath)
		$sInclude_Before = $sIncludePath

		$sSection = '#include <' & $sInclude_After & '>' & $CRLF

		; Add params fixing.
		If _HelpCheck_SyntaxParams($aSection[$SECTION_PARAMS], $aFuncParams) Then
			$iError += $HELPCHECK_ERROR_SYNTAX_PARAMSFORMAT
		EndIf

		$sSection &= $aSection[$SECTION_PARAMS]
		$bReturn = Not ($sSection == $sSectionOrginal)
		If $bReturn Then
			$iError += $HELPCHECK_ERROR_SYNTAX_INCLUDEFORMAT
			_HelpFile_SetSection($hHelpFile, $HELP_SECTION_SYNTAX, $sSection)
		EndIf
	EndIf

	Return SetError($iError, 0, $bReturn)
EndFunc   ;==>_HelpCheck_Syntax

Func _HelpCheck_SyntaxParams(ByRef $sData, ByRef $aFuncParams) ; $aFuncParams to be used when checking if the parameters match the syntax line.
	Local $bReturn = False

	; Set the out parameters.
	$aFuncParams = 0

	Local $sDataOriginal = $sData

	; If not "FuncName (" then return False.
	If Not StringRegExp($sData, '^\h*\w+\h*\(') Then Return $bReturn

	; Get the function name.
	Local $aFuncName = StringRegExp($sData, '^\w+', $STR_REGEXPARRAYGLOBALMATCH)
	If @error Then Return $bReturn
	Local $sFuncName = $aFuncName[0]

	$bReturn = True

	; Store string literals temporarily.
	Local $aStrings = StringRegExp($sData, '(([''"])\V*?\2)', $STR_REGEXPARRAYGLOBALMATCH)
	For $i = 0 To UBound($aStrings) - 1 Step 2
		$sData = StringReplace($sData, $aStrings[$i], $ASCII_ACK, $STR_REPLACE_ONCE, $STR_CASESENSE)
	Next

	Local $sParams = StringRegExpReplace($sData, '(?:^\w*\h*\(\h*\[?|\h*\]*\h*\)\h*$)', '') ; Strip FuncName ( [ ... AND/OR .... ]]]] )
	Local $iBracket = 0, $iIndex = 0, $iParamIndex = $HELPCHECK_PARAMS_VARIABLE

	StringReplace($sParams, ',', ',')
	Local $aParams[@extended + 1][$HELPCHECK_PARAMS_MAX]
	$aFuncParams = $aParams

	Local $iBrackets = 0, _
			$sChr = '', $sParamString = ''
	For $i = 1 To StringLen($sParams)
		$sChr = StringMid($sParams, $i, 1)
		Switch $sChr
			Case '('
				$iBracket += 1
			Case ')'
				$iBracket -= 1
			Case ','
				If Not $iBracket Then
					$aFuncParams[$iIndex][$iParamIndex] = StringRegExpReplace($aFuncParams[$iIndex][$iParamIndex], '\h*\[$', '')
					$iParamIndex = $HELPCHECK_PARAMS_VARIABLE
					$iIndex += 1
					ContinueLoop
				EndIf
			Case '='
				If $iParamIndex = $HELPCHECK_PARAMS_VARIABLE Then
					$aFuncParams[$iIndex][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_ISOPTIONALPARAMS
					$iParamIndex = $HELPCHECK_PARAMS_VALUEORDESC
					$sChr = ''
				EndIf
		EndSwitch
		$aFuncParams[$iIndex][$iParamIndex] &= $sChr
	Next
	ReDim $aFuncParams[$iIndex + 1][$HELPCHECK_PARAMS_MAX]

	$iBrackets = 0 ; Reset the brackets count.
	Local $bFirstIteration = True, _
			$iMaxParams = -1, _
			$sVariable = ''
	For $i = 0 To UBound($aFuncParams) - 1 ; Tidy up the param array.
		$sVariable = StringStripWS($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE], $STR_WHITESPACE) ; Strip white-space.
		$aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC] = StringStripWS($aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC], $STR_WHITESPACE) ; Strip white-space.

		If $bFirstIteration And $aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC] Then
			$sParamString &= '['
		EndIf
		$sParamString &= ((Not $bFirstIteration And $aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC]) ? ' [' : '') & ', ' & $sVariable & _
				($aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC] ? ' = ' : '') & $aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC]
		If $aFuncParams[$i][$HELPCHECK_PARAMS_VALUEORDESC] Then
			$iBrackets += 1
		EndIf
		$aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE] = StringRegExpReplace($sVariable, '^.*(\$\w+).*$', '\1') ; Strip everything but the variable name.

		; Check if valid though undocumented parameters.
		If $iMaxParams = -1 And StringLeft($aFuncParams[$i][$HELPCHECK_PARAMS_VARIABLE], StringLen('$_iCurrent')) = '$_iCurrent' Then ; This is an inline workaround for functions that contain variables that begin with $_iCurrent***.
			$iMaxParams = $i
		EndIf
		$bFirstIteration = False
	Next

	; Ignore valid undocumented parameters
	If $iMaxParams = 0 Then
		$aFuncParams = 0 ; If no parameters then set to zero.
	ElseIf $iMaxParams > 0 Then
		ReDim $aFuncParams[$iMaxParams][$HELPCHECK_PARAMS_MAX]
	EndIf

	; Strip first comma or bracket.
	$sParamString = StringStripWS($sParamString, $STR_WHITESPACE)
	If StringLeft($sParamString, StringLen('[, ')) == '[, ' Then
		$sParamString = '[' & StringTrimLeft($sParamString, StringLen('[, '))
	EndIf
	If StringLeft($sParamString, StringLen(', ')) = ', ' Then
		$sParamString = StringTrimLeft($sParamString, StringLen(', '))
	EndIf
	; Add closing brackets.
	$sParamString = $sParamString & ($iBrackets ? _StringRepeat(']', $iBrackets) : '')

	; If an no parameters.
	If $sParamString = ',' Then
		$sParamString = ''
	EndIf

	; Re-build string literals using stored array.
	For $i = 0 To UBound($aStrings) - 1 Step 2
		$sParamString = StringReplace($sParamString, $ASCII_ACK, $aStrings[$i], $STR_REPLACE_ONCE, $STR_CASESENSE)
		For $j = 0 To UBound($aFuncParams) - 1
			$aFuncParams[$j][$HELPCHECK_PARAMS_VALUEORDESC] = StringReplace($aFuncParams[$j][$HELPCHECK_PARAMS_VALUEORDESC], $ASCII_ACK, $aStrings[$i], $STR_REPLACE_ONCE, $STR_CASESENSE)
			If @extended Then ExitLoop
		Next
	Next

	If StringStripWS($sParamString, $STR_STRIPALL) == '' Then
		$aFuncParams = 0 ; If no parameters then set to zero.
	EndIf

	$sData = $sFuncName & ' ( ' & $sParamString & ' )'
	$bReturn = Not ($sDataOriginal == $sData)
	Return $bReturn
EndFunc   ;==>_HelpCheck_SyntaxParams

Func _HelpCheck_SyntaxTag($sData, ByRef $aTagParams) ; Convert a $tag syntax line into a list of "variables" to check.
	$sData = StringRegExpReplace($sData, '(?i:^Global\h+Const\h+\$\w+\h+\=\h+)', '') ; Strip declaration part.
	$sData = StringRegExpReplace($sData, '(?:(?<=;)\h+)', '') ; Strip white-space after a semi-colon.
	$sData = StringRegExpReplace($sData, '(?i:\balign\b\h+\d+(?:;|$))', ';') ; Strip align 4;.
	$sData = StringRegExpReplace($sData, '(?:(?<=[;"''])\w+\h+)', '') ; Strip the datatype e.g. ;word
	$sData = StringRegExpReplace($sData, '(?:["''&_\s]|\[\d+\])', '') ; Strip quotes, ampersand, underscore and white-space. Also array accessing brackets.
	$sData = StringRegExpReplace($sData, '(?:;?(?:end)?struct;?)', ';') ; Strip struct and/or endstruct.
	$sData = StringRegExpReplace($sData, '^;*(.+?);*$', '\1') ; Strip colons at the start and end of a line.
	$sData = StringRegExpReplace($sData, ';{2,}', ';') ; Strip double semi-colons e.g. ;;.

	If StringStripWS($sData, $STR_STRIPALL) = '' Then
		$aTagParams = 0
	Else
		$aTagParams = StringSplit($sData, ';', $STR_STRINGSPLIT)
		Local $iUBound = UBound($aTagParams)
		Local $aTags[$iUBound][$HELPCHECK_PARAMS_MAX]
		For $i = 0 To $iUBound - 1
			$aTags[$i][$HELPCHECK_PARAMS_VARIABLE] = $aTagParams[$i]
			$aTags[$i][$HELPCHECK_PARAMS_VALUEORDESC] = ''
			$aTags[$i][$HELPCHECK_PARAMS_OPTIONAL] = $HELPCHECK_PARAMS_OPTIONAL_ISNOTOPTIONALPARAMS
		Next
		$aTagParams = $aTags
	EndIf

	Return True
EndFunc   ;==>_HelpCheck_SyntaxTag

Func _HelpCheck_Title(ByRef $hHelpFile)
	Local Static $sAlternativeTitles = 0
	If Not IsString($sAlternativeTitles) Then
		$sAlternativeTitles = $CRLF & FileRead(@ScriptDir & '\title_checking.txt') & $CRLF
	EndIf
	Local $bReturn = False
	If _HelpFile_IsHelpFile($hHelpFile) Then
		Local $sFuncName = _HelpFile_GetFileName($hHelpFile), _
				$sTitle = StringRegExp($sAlternativeTitles & $sFuncName & '->' & $sFuncName & $CRLF, '(?m:^\Q' & $sFuncName & '\E\h*\->\h*(\V+)$)', $STR_REGEXPARRAYGLOBALMATCH)[0] ; Check if an alternative exists in title_checking.txt, else default to the current filename.
		$bReturn = StringRegExp(_HelpFile_GetBuildData($hHelpFile), '(?:^###[^#]+?###\R\Q' & $sTitle & '\E\R)') > 0 ; Check if ###Section###CRLFFuncName exists
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpCheck_Title

Func _HelpCheck_URLFormatting(ByRef $hHelpFile)
	Local $aLinksErrors[0], _
			$hLinksErrors = 0, _
			$iExtended = 0
	If _HelpFile_IsHelpFile($hHelpFile) And _AssociativeArray_Startup($hLinksErrors, True) Then
		Local $sData = _HelpFile_GetBuildData($hHelpFile)
		Local $aLinks = StringRegExp($sData, '(?:href=(["''])(.*?)\1)', $STR_REGEXPARRAYGLOBALMATCH), _
				$sHTML = '', $sTXT = '', $sURL = ''
		For $i = 0 To UBound($aLinks) - 1 Step 2
			$sURL = $aLinks[$i + 1]
			If $sURL Then
				$sURL = StringReplace($sURL, '\', '/')
				$iExtended += @extended
				$sData = StringReplace($sData, $sURL, $sURL, $STR_REPLACE_ONCE, $STR_CASESENSE) ; Format links to use forward slash(s) instead of back slash(s).
				If Not _WinAPI_UrlIs($sURL) And Not StringInStr($sURL, 'Management.htm') Then
					$sHTML = $sURL

					$sHTML = StringRegExpReplace($sHTML, '#.*$', '') ; IDs.
					$sHTML = StringReplace($sHTML, '../', '')
					$sHTML = StringReplace($sHTML, 'libfunctions/', '')
					$sHTML = StringReplace($sHTML, 'functions/', '')
					$sHTML = StringReplace($sHTML, 'keywords/', '')

					$sTXT = StringRegExpReplace($sHTML, '\.htm(?:l)?$', '.txt')

					$sHTML = _WinAPI_PathFindOnPath($sHTML, $PFOP_PATHS)
					$sTXT = _WinAPI_PathFindOnPath($sTXT, $PFOP_PATHS)

					If Not FileExists($sTXT) And Not FileExists($sHTML) Then
						$hLinksErrors.Item($sURL) = $sURL
					EndIf
				EndIf
			EndIf
		Next
		$aLinksErrors = $hLinksErrors.Items()
	EndIf
	Return SetError(UBound($aLinksErrors) + $iExtended, $iExtended, $aLinksErrors)
EndFunc   ;==>_HelpCheck_URLFormatting

Func __HelpCheck_GetParametersTable(ByRef $sSection, ByRef $sSectionOriginal, ByRef $sTable)
	; Set the out parameters.
	$sSectionOriginal = ''
	$sTable = ''

	Local Enum $HELPCHECK_PARAMS_TABLE_FULL, _
			$HELPCHECK_PARAMS_TABLE, _
			$HELPCHECK_PARAMS_TABLE_MAX
	Local $bReturn = False, _
			$aArray = StringRegExp($sSection, '(?is:^.*(' & $HELP_TABLE_ID & 'ParamTable' & $HELP_TABLE_ID & '\R(.*?)\R' & $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID & ').*)', $STR_REGEXPARRAYGLOBALMATCH)
	If Not @error And UBound($aArray) = $HELPCHECK_PARAMS_TABLE_MAX Then
		$bReturn = True
		$sSectionOriginal = $aArray[$HELPCHECK_PARAMS_TABLE_FULL]
		$sTable = $aArray[$HELPCHECK_PARAMS_TABLE]
	EndIf
	Return $bReturn
EndFunc   ;==>__HelpCheck_GetParametersTable
#EndRegion Tidy NATIVE/USERDEFINED/KEYWORD TXT Docs

#Region Misc checking functions
Func _HelpMisc_CategoriesTOC(ByRef Const $aArray, $sCategoriesPath)
	Local $bIsError = False, _
			$iModifiedTime = FileGetTime($sCategoriesPath, $FT_MODIFIED, $FT_STRING)
	Local $sData = FileRead($sCategoriesPath)
	If $sData = '' Or FileGetTime($sCategoriesPath, $FT_CREATED, $FT_STRING) = $iModifiedTime Then Return $bIsError

	ConsoleWrite('Categories.toc checking...' & $CRLF)

	Local $hFileListFunctions = 0, $hCategoryFunctions = 0
	If _AssociativeArray_Startup($hCategoryFunctions, False) And _AssociativeArray_Startup($hFileListFunctions, False) Then
		Local $sFuncName = ''
		For $i = 1 To UBound($aArray) - 1
			$sFuncName = _WinAPI_PathRemoveExtension(_WinAPI_PathStripPath($aArray[$i]))
			$hFileListFunctions.Item($sFuncName) = $sFuncName ; Add FuncName to the dictionary.
		Next

		Local $sDataOriginal = $sData
		_ConvertEOLToCRLF($sData)
		_StripTrailingWhitespace($sData)

		If _FileWrite_IfDiff($sCategoriesPath, $sCategoriesPath, $sDataOriginal, $sData) Then
			ConsoleWrite('! Removed unnecessary white-space from Categories.toc.' & $CRLF)
		EndIf

		Local $aFuncs = StringRegExp($sData & $CRLF, '([^|:\r\n]+)(?=\R|$)', $STR_REGEXPARRAYGLOBALMATCH) ; Match function names at the end of line e.g. $tag... or _FuncName_.
		For $i = 0 To UBound($aFuncs) - 1
			If $hCategoryFunctions.Exists($aFuncs[$i]) Then ; Check for duplicates in the Categories.toc.
				$bIsError = True
				ConsoleWrite('! ' & $aFuncs[$i] & ' is already mentioned in the Categories.toc file.' & $CRLF)
			Else
				$hCategoryFunctions.Item($aFuncs[$i]) = $aFuncs[$i] ; Add FuncName to the dictionary.
			EndIf
		Next

		$aFuncs = $hFileListFunctions.Items() ; Check if files are listed in the txt file directory but not the Categories.toc.
		For $i = 0 To UBound($aFuncs) - 1
			If Not $hCategoryFunctions.Exists($aFuncs[$i]) Then
				$bIsError = True
				ConsoleWrite('! ' & $aFuncs[$i] & ' exists in the txt file directory, but not Categories.toc.' & $CRLF)
			EndIf
		Next

		$aFuncs = $hCategoryFunctions.Items() ; Check if files are listed in Categories but not the txt file directory.
		For $i = 0 To UBound($aFuncs) - 1
			If Not $hFileListFunctions.Exists($aFuncs[$i]) Then
				$bIsError = True
				ConsoleWrite('! ' & $aFuncs[$i] & ' exists in Categories.toc, but not the txt file directory.' & $CRLF)
			EndIf
		Next

		If Not $bIsError Then
			FileSetTime($sCategoriesPath, $iModifiedTime, $FT_CREATED)
		EndIf
	EndIf
	ConsoleWrite('Categories.toc checking finished.' & $CRLF & $CRLF)

	Return $bIsError
EndFunc   ;==>_HelpMisc_CategoriesTOC

Func _HelpMisc_IncludeFunctions(ByRef Const $aArray, ByRef $sIncludeParams)
	; Set the out parameter.
	$sIncludeParams = ''

	ConsoleWrite('Include functions checking...' & $CRLF)

	Local $bIsError = False, _
			$hFileListFunctions = 0, $hIncludeFunctions = 0
	If _AssociativeArray_Startup($hFileListFunctions, False) And _AssociativeArray_Startup($hIncludeFunctions, False) Then
		If UBound($aArray) Then
			Local $sFuncName = ''
			For $i = 1 To UBound($aArray) - 1
				$sFuncName = _WinAPI_PathStripPath(_WinAPI_PathRemoveExtension($aArray[$i]))
				If Not StringRegExp($sFuncName, '^\$') Then ; If not a $tag constant.
					$hFileListFunctions.Item($sFuncName) = $sFuncName ; Add FuncName to the dictionary.
				EndIf
			Next
		EndIf

		Local $aFuncs = 0
		Local $aIncludes = _FileListToArray($INCLUDESOURCE, '*.au3', $FLTA_FILES, True)
		If Not @error Then
			Local $sData = ''
			For $i = 1 To UBound($aIncludes) - 1
				$sData &= FileRead($aIncludes[$i])
			Next

			; Prepare the merged includes by stripping comments and merging continuation lines.
			_StripCommentLines($sData)
			_StripMerge($sData)

			$sData = StringRegExpReplace($sData, '(?im:^(?!\bFunc\b\h_[A-Z])\V*\R)', '') ; Strip everything but Func FuncName(params). Doesn't include internal functions.
			$sIncludeParams = StringRegExpReplace($sData, '(?i:(?:^|\R)Func\h)', $CRLF) ; Strip Func keywords and internal functions.

			_GetFunctionInfo($sData, $aFuncs)
			For $i = 0 To UBound($aFuncs) - 1
				If $hIncludeFunctions.Exists($aFuncs[$i]) Then ; Check for duplicates in the includes.
					$bIsError = True
					ConsoleWrite('! ' & $aFuncs[$i] & ' is already mentioned in the include files.' & $CRLF)
				Else
					$hIncludeFunctions.Item($aFuncs[$i]) = $aFuncs[$i] ; Add FuncName to the dictionary.
				EndIf
			Next
			$sData = ''
		EndIf
		$aIncludes = 0

		Local $aIgnoreFunctions = FileReadToArray(@ScriptDir & '\undocumented_functions.txt') ; Add the ignored functions to both dictionaries.
		For $i = 0 To UBound($aIgnoreFunctions) - 1
			$hFileListFunctions.Item($aIgnoreFunctions[$i]) = $aIgnoreFunctions[$i] ; Add FuncName to the dictionary.
			$hIncludeFunctions.Item($aIgnoreFunctions[$i]) = $aIgnoreFunctions[$i] ; Add FuncName to the dictionary.
		Next

		$aFuncs = $hFileListFunctions.Items() ; Check if files are listed in the txt file directory but not the includes.
		For $i = 0 To UBound($aFuncs) - 1
			If Not $hIncludeFunctions.Exists($aFuncs[$i]) Then
				$bIsError = True
				ConsoleWrite('! ' & $aFuncs[$i] & ' exists in the txt file directory, but not include files.' & $CRLF)
			EndIf
		Next

		$aFuncs = $hIncludeFunctions.Items() ; Check if files are listed in includes but not the txt file directory.
		For $i = 0 To UBound($aFuncs) - 1
			If Not $hFileListFunctions.Exists($aFuncs[$i]) Then
				$bIsError = True
				ConsoleWrite('! ' & $aFuncs[$i] & ' exists in include files, but not the txt file directory.' & $CRLF)
			EndIf
		Next
	EndIf
	ConsoleWrite('Include functions checking finished.' & $CRLF & $CRLF)

	Return $bIsError
EndFunc   ;==>_HelpMisc_IncludeFunctions
#EndRegion Misc checking functions
