; ===================================================================
; Project: Help File Misc
; Description: This file contains misc TXT doc related functions that can be shared between the checking and generation scripts.
; Author: guinness
; ===================================================================
#include-once

#include 'HelpFileLib.au3' ; HelpFileLib by guinness.

Global Const $HELPMISC_ERROR_NONE = 0
Global Enum Step * 2 $HELPMISC_ERROR_EMPTYSECTION, _
		$HELPMISC_ERROR_INCLUDE_INVALIDSECTION

Func _HelpMisc_GetIncludeFileName(ByRef $hHelpFile)
	Local $iError = $HELPMISC_ERROR_NONE, _
			$sFileName = _HelpFile_GetFileName($hHelpFile)
	Local $sSection = _HelpFile_GetSection($hHelpFile, '###Example###')
	If @error Then
		$iError = $HELPMISC_ERROR_INCLUDE_INVALIDSECTION
	Else
		If StringStripWS($sSection, $STR_STRIPALL) = '' Then
			$iError += $HELPMISC_ERROR_EMPTYSECTION
		Else
			$sFileName = _HelpMisc_GetIncludeFileNameFromData($sSection, $sFileName)
		EndIf
	EndIf

	Return SetError($iError, 0, $sFileName)
EndFunc   ;==>_HelpMisc_GetIncludeFileName

Func _HelpMisc_GetIncludeFileNameFromData(ByRef $sData, $sFileName)
	; Retrieve the include duplicate function. If one doesn't exist then it uses the filename of the TXT doc. The section checking determines whether or not to include the example(s).
	Return StringRegExp($sData & @CRLF & _
			'@@IncludeExample@@ ' & $sFileName & @CRLF, '(?i:@@IncludeExample@@\h*([$@\w\-]+)\h*\R)', $STR_REGEXPARRAYGLOBALMATCH)[0] ; Find the duplicate function e.g. @@IncludeExample@@ SomeFunc
EndFunc   ;==>_HelpMisc_GetIncludeFileNameFromData

Func _HelpMisc_GetIncludeFilePath(ByRef $hHelpFile)
	Local $sFilePath = _HelpFile_GetExamplePath($hHelpFile) & '\' & _HelpMisc_GetIncludeFileName($hHelpFile) & '.' & (_HelpFile_IsAutoItX($hHelpFile) ? 'vbs' : 'au3')
	Return SetError(@error, 0, $sFilePath)
EndFunc   ;==>_HelpMisc_GetIncludeFilePath

Func _HelpMisc_IsNone($sData)
	Return StringRegExp($sData, '(?i:^\h*\bNone\b\.?\h*$)') > 0
EndFunc   ;==>_HelpMisc_IsNone
