#include '..\gen_editorfuncs.au3'

GenerateSyntax()

; ------------------------------------------------------------------------------
; Automatically generate the php file.
; ------------------------------------------------------------------------------
Func GenerateSyntax()
	Local $sOutputFilePath = @ScriptDir & '\autoit.php'
	Local $hFileOpen = FileOpen($sOutputFilePath, $FO_OVERWRITE_CREATE)
	If $hFileOpen = -1 Then Return False

	Local $sHeader = FileRead(@ScriptDir & '\header.txt')
	; Update the copyright year(s) field.
	$sHeader = StringRegExpReplace($sHeader, '(\(c\)\h+\d{4}-)\d{4}', '${1}' & @YEAR)

	; Update the AutoIt version field.
	Local $sVersion = IniRead('..\..\..\..\..\_build\config.ini', 'Build', 'AutoItVersion', @AutoItVersion) ; Retrieves the AutoIt version.
	$sHeader = StringRegExpReplace($sHeader, '(AutoIt:\h+v)\V+', '${1}' & $sVersion)

	; Update the updated field.
	$sHeader = StringRegExpReplace($sHeader, '(Updated:\h+)(\d{4})/(\d{2})/(\d{2})', '${1}' & @YEAR & '/' & @MON & '/' & @MDAY)
	FileWrite($hFileOpen, $sHeader)

	Local $aKeywords = 0, $aComments = 0, $aPreProcessor = 0, $aPragma = 0, $aAu3Check = 0, $aRegion = 0
	_SplitKeywords($KEYWORDLIST, $aKeywords, $aComments, $aPreProcessor, $aPragma, $aAu3Check, $aRegion)
	_AppendPHPArray($hFileOpen, '1', $aKeywords)

	Local $aMacros = FileReadToArray($MACROLIST)
	_AppendPHPArray($hFileOpen, '2', $aMacros)

	Local $aFunctions = FileReadToArray($FUNCTIONLIST)
	_ArraySort($aFunctions)
	_AppendPHPArray($hFileOpen, '3', $aFunctions)

	Local $aLibFunctions = FileReadToArray($LIBFUNCTIONLIST)
	_StripFirstChar($aLibFunctions)
	_AppendPHPArray($hFileOpen, '4', $aLibFunctions)

	_AppendPHPArray($hFileOpen, '5', $aComments)

	Local $aSciteDirectives = FileReadToArray(@ScriptDir & '\SciTEDirectives.txt')
	ArrayJoin($aSciteDirectives, $aRegion)
	_AppendPHPArray($hFileOpen, '6', $aSciteDirectives)

	Local $aSendKeys = FileReadToArray($SENDSTRINGLIST)
	_StripFirstAndLastChar($aSendKeys)
	_AppendPHPArray($hFileOpen, '7', $aSendKeys)

	_AppendPHPArray($hFileOpen, '8', $aPreProcessor)
	_AppendPHPArray($hFileOpen, '9', $aPragma)
	_AppendPHPArray($hFileOpen, '10', $aAu3Check)

	Local $aFooters = FileReadToArray(@ScriptDir & '\footer.txt')
	For $i = 0 To UBound($aFooters) - 1
		FileWriteLine($hFileOpen, $aFooters[$i])
	Next

	FileClose($hFileOpen)

	Return True
EndFunc   ;==>GenerateSyntax

Func _AppendPHPArray(ByRef $hFileOpen, $iValue, ByRef $aArray)
	FileWriteLine($hFileOpen, _AddBlanks(8) & $iValue & ' => array(')
	Local $sTemp = _AddBlanks(12)

	For $i = 0 To UBound($aArray) - 1
		_StringClip($hFileOpen, $aArray[$i], $sTemp)
	Next

	_StringEnd($hFileOpen, $sTemp)

	Return True
EndFunc   ;==>_AppendPHPArray

Func _AddBlanks($iLength)
	Local $sReturn = ''
	For $i = 1 To $iLength
		$sReturn &= ' '
	Next
	Return $sReturn
EndFunc   ;==>_AddBlanks

Func _SplitKeywords($sFilePath, ByRef $aKeywords, ByRef $aComments, ByRef $aPreProcessor, ByRef $aPragma, ByRef $aAu3Check, ByRef $aRegion)
	Local $sData = FileRead($sFilePath)

	; Keywords.
	$aKeywords = StringRegExp($sData, '(?m)^(\w+)', $STR_REGEXPARRAYGLOBALMATCH)
	_StripKeywordFromData($sData, $aKeywords)

	; Comments.
	$aComments = StringRegExp($sData, '(?im)^#(c(?:e|s)|comments-(?:end|start))', $STR_REGEXPARRAYGLOBALMATCH)
	_StripKeywordFromData($sData, $aComments, '#')

	; Pragma.
	$aPragma = StringRegExp($sData, '(?im)^#(pragma)', $STR_REGEXPARRAYGLOBALMATCH)
	_StripKeywordFromData($sData, $aPragma, '#')

	; Au3Check.
	$aAu3Check = StringRegExp($sData, '(?im)^#(force(?:def|ref)|ignorefunc)', $STR_REGEXPARRAYGLOBALMATCH)
	_StripKeywordFromData($sData, $aAu3Check, '#')

	; Regions.
	$aRegion = StringRegExp($sData, '(?im)^#((?:End)?Region)', $STR_REGEXPARRAYGLOBALMATCH)
	_StripKeywordFromData($sData, $aRegion, '#')

	; PreProcessor.
	$aPreProcessor = StringRegExp($sData, '(?m)^#([\w\-]+)', $STR_REGEXPARRAYGLOBALMATCH)
	_StripKeywordFromData($sData, $aPreProcessor, '#')

	Return True
EndFunc   ;==>_SplitKeywords

Func _StripKeywordFromData(ByRef $sData, ByRef $aArray, $sPreAppend = '')
	For $i = 0 To UBound($aArray) - 1
		$sData = StringReplace(@CRLF & $sData & @CRLF, @CRLF & $sPreAppend & $aArray[$i] & @CRLF, @CRLF)
	Next

	Return True
EndFunc   ;==>_StripKeywordFromData

Func _StringClip(ByRef $hFileOpen, $sText, ByRef $sTempString, $iLength = 80)
	$sText = "'" & $sText & "',"
	If StringLen($sTempString & $sText) > $iLength Then
		FileWriteLine($hFileOpen, $sTempString)
		$sTempString = _AddBlanks(12) & $sText
	Else
		$sTempString &= $sText
	EndIf

	Return True
EndFunc   ;==>_StringClip

Func _StringEnd(ByRef $hFileOpen, ByRef Const $sTempString)
	Local $iLength = 1
	FileWriteLine($hFileOpen, StringTrimRight($sTempString, $iLength))
	FileWriteLine($hFileOpen, _AddBlanks(12) & '),')

	Return True
EndFunc   ;==>_StringEnd

Func _StripFirstChar(ByRef $aArray)
	Local $iLength = 1
	For $i = 0 To UBound($aArray) - 1
		$aArray[$i] = StringTrimLeft($aArray[$i], $iLength)
	Next

	Return True
EndFunc   ;==>_StripFirstChar

Func _StripFirstAndLastChar(ByRef $aArray)
	Local $iLength = 1
	For $i = 0 To UBound($aArray) - 1
		$aArray[$i] = StringTrimRight(StringTrimLeft($aArray[$i], $iLength), $iLength)
	Next

	Return True
EndFunc   ;==>_StripFirstAndLastChar
