#include '..\gen_editorfuncs.au3'

GenerateSyntax()

; ------------------------------------------------------------------------------
; Automatically generate the syntax file(s).
; ------------------------------------------------------------------------------
Func GenerateSyntax()
	Local $aKeywords = FileReadToArray($KEYWORDLIST), _
			$aMacros = FileReadToArray($MACROLIST), _
			$aFunctions = FileReadToArray($FUNCTIONLIST), _
			$aLibFunctions = FileReadToArray($LIBFUNCTIONLIST), _
			$aSendStrings = FileReadToArray($SENDSTRINGLIST)

	Local Const $sHeader = @ScriptDir & '\header.txt', _
			$sOutputFilePath = @ScriptDir & '\AutoIt.tmLanguage'

	; Copy the header.txt file to the output file.
	FileCopy($sHeader, $sOutputFilePath, $FC_OVERWRITE)

	Local Const $sAppend = '|'

	; Keywords
	Local $iStringLen = StringLen('#')
	Local $sDirectiveList = '', $sKeywordList = ''
	For $i = 0 To UBound($aKeywords) - 1
		If StringLeft($aKeywords[$i], $iStringLen) = '#' Then
			$sDirectiveList &= $aKeywords[$i] & $sAppend
		Else
			$sKeywordList &= $aKeywords[$i] & $sAppend
		EndIf
	Next
	; Strip #ce, #comments-end, #cs, #comments-start and #include from the directive list.
	$sDirectiveList = StringRegExpReplace($sDirectiveList, '(?:#comments-(?:end|start)\||#c(?:e|s)\||#include(?!\-once)\|)', '')
	$sDirectiveList = StringReplace($sDirectiveList, '#', '')

	; Macros
	Local $sMacroList = ''
	For $i = 0 To UBound($aMacros) - 1
		$sMacroList &= $aMacros[$i] & $sAppend
	Next
	$sMacroList = StringReplace($sMacroList, '@', '')

	; Functions
	Local $sFunctionList = ''
	For $i = 0 To UBound($aFunctions) - 1
		$sFunctionList &= $aFunctions[$i] & $sAppend
	Next

	; LibFunctions
	Local $sLibFunctionList = ''
	For $i = 0 To UBound($aLibFunctions) - 1
		$sLibFunctionList &= $aLibFunctions[$i] & $sAppend
	Next

	; SendStrings
	Local $sSendStrings = ''
	For $i = 0 To UBound($aSendStrings) - 1
		$sSendStrings &= $aSendStrings[$i] & $sAppend
	Next
	$sSendStrings = StringRegExpReplace($sSendStrings, '[{}]', '')

	; Convert to lowercase and strip the last dilimiter.
	$iStringLen = StringLen('|')
	$sDirectiveList = StringTrimRight(StringLower($sDirectiveList), $iStringLen)
	$sFunctionList = StringTrimRight(StringLower($sFunctionList), $iStringLen)
	$sKeywordList = StringTrimRight(StringLower($sKeywordList), $iStringLen)
	$sLibFunctionList = StringTrimRight(StringLower($sLibFunctionList), $iStringLen)
	$sMacroList = StringTrimRight(StringLower($sMacroList), $iStringLen)
	$sSendStrings = StringTrimRight(StringLower($sSendStrings), $iStringLen)

	; Create Sublime Text editor list.
	Local $sData = FileRead($sOutputFilePath)

	$sData = StringReplace($sData, '__DIRECTIVES__', $sDirectiveList)
	$sData = StringReplace($sData, '__NATIVEFUNCTIONS__', $sFunctionList)
	$sData = StringReplace($sData, '__KEYWORDS__', $sKeywordList)
	$sData = StringReplace($sData, '__UDFFUNCTIONS__', $sLibFunctionList)
	$sData = StringReplace($sData, '__MACROS__', $sMacroList)
	$sData = StringReplace($sData, '__SENDSTRINGS__', $sSendStrings)

	Return _FileWrite($sOutputFilePath, $sData)
EndFunc   ;==>GenerateSyntax
